import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter_waya/src/constant/WayColor.dart';
import 'package:flutter_waya/src/constant/WayIcon.dart';
import 'package:flutter_waya/src/constant/WayStyles.dart';
import 'package:flutter_waya/src/tools/Tools.dart';
import 'package:flutter_waya/src/widget/custom/CustomFlex.dart';

class ListItem extends StatelessWidget {
  final GestureTapCallback onTap;
  final GestureTapCallback onDoubleTap;
  final GestureLongPressCallback onLongPress;
  final bool isThreeLine; //是否默认3行高度，subtitle不为空时才能使用
  final bool selected;
  final bool dense; //设置为true后字体变小
  final EdgeInsetsGeometry contentPadding;
  final Widget leading; //左侧widget
  final Widget subtitle; //副标题
  final Widget child; //右侧widget
  final Widget title;
  final String titleText;
  final TextStyle titleStyle;
  final double height;
  final Widget prefix;

  final EdgeInsetsGeometry padding;
  final EdgeInsetsGeometry margin;
  final EdgeInsetsGeometry prefixMargin;
  final bool inkWell;
  final Color underlineColor;

  final EdgeInsetsGeometry arrowMargin;
  final Color backgroundColor;
  final BoxDecoration decoration;
  final double arrowSize;
  final Color arrowColor;
  final Widget arrowIcon;
  final bool arrow;
  final bool enabled;

  ListItem({Key key,
    double arrowSize,
    Color arrowColor,
    bool isThreeLine,
    bool arrow,
    bool enabled,
    this.onTap,
    this.onDoubleTap,
    this.onLongPress,
    this.title,
    this.height,
    this.inkWell,
    this.padding,
    this.margin,
    this.decoration,
    this.child,
    this.backgroundColor,
    this.titleText,
    this.titleStyle,
    this.underlineColor,
    this.leading,
    this.subtitle,
    this.dense,
    this.contentPadding,
    this.selected,
    this.arrowIcon,
    this.arrowMargin,
    this.prefix,
    this.prefixMargin})
      : this.arrowSize = arrowSize ?? Tools.getWidth(16),
        this.arrowColor = arrowColor ?? getColors(black),
        this.isThreeLine = isThreeLine ?? false,
        this.arrow = arrow ?? true,
        this.enabled = enabled ?? true,
        super(key: key);

  @override
  Widget build(BuildContext context) {
    return CustomFlex(
      height: height,
      inkWell: inkWell,
      margin: margin,
      padding: padding,
      onLongPress: enabled ? onLongPress : null,
      onDoubleTap: enabled ? onDoubleTap : null,
      onTap: enabled ? onTap : null,
      direction: Axis.horizontal,
      mainAxisAlignment: MainAxisAlignment.center,
//      crossAxisAlignment: CrossAxisAlignment.center,
      decoration:
      decoration ?? WayStyles.containerUnderlineBackground(
          underlineColor: underlineColor, color: backgroundColor),
      children: <Widget>[
        Container(margin: prefixMargin, child: prefix),
        Expanded(
            child: ListTile(
              contentPadding: contentPadding,
              title: title ?? Text(titleText, style: titleStyle),
              subtitle: subtitle,
              leading: leading,
              trailing: child,
              isThreeLine: isThreeLine,
              dense: dense,
              enabled: false,
              selected: false, //展示是否默认显示选中
            )),
        Offstage(
          offstage: !arrow,
          child: Container(
              margin: arrowMargin,
              child: arrowIcon ??
                  Icon(
                    WayIcon.arrowRight,
                    size: arrowSize,
                    color: arrowColor,
                  )),
        ),
      ],
    );
  }
}
