import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_waya/flutter_waya.dart';
import 'package:flutter_waya/src/constant/WayColor.dart';
import 'package:flutter_waya/src/constant/WayConstant.dart';
import 'package:flutter_waya/src/constant/WayEnum.dart';
import 'package:flutter_waya/src/constant/WayStyles.dart';

class TextInputField extends StatelessWidget {
  final TextEditingController controller;
  final ValueChanged<String> onChanged;
  final List<TextInputFormatter> inputFormatter;
  final int maxLength; //最大长度
  final TextDirection textDirection;
  final int maxLines;
  final TextAlign textAlign; // 对齐方式

  final int minLines;
  final TextInputType keyboardType;
  final InputDecoration inputDecoration;
  final Color cursorColor; //光标颜色
  final InputTextType inputTextType;

  //点击事件
  final GestureTapCallback onTap;
  final VoidCallback onEditingComplete;
  final ValueChanged<String> onSubmitted;

  //显示在输入框右下方
  final String counterText;
  final TextStyle counterStyle;
  final Widget counter;

  final bool maxLengthEnforced; // //是否自动获取焦点  跳转到该页面后 光标自动显示到该输入框  键盘弹起
  final bool autoFocus; // //是否自动获取焦点  跳转到该页面后 光标自动显示到该输入框  键盘弹起
  final bool obscureText; //隐藏文字
  final Widget header;
  final Widget footer;
  final FocusNode focusNode;
  final TextStyle inputStyle;

  //输入框左边下划线外面
  final Widget icon;

  //输入框左边
  final Widget prefixIcon; //前缀
  final String prefixText;
  final Widget prefix;
  final TextStyle prefixStyle;

  //输入框右边
  final Widget suffixIcon; //后缀
  final String suffixText;
  final Widget suffix;
  final TextStyle suffixStyle;

  //显示在输入框左下角的文字提示
  final TextStyle errorStyle;
  final String errorText;
  final int errorMaxLines;

  //显示在输入框下划线下面的提示语，提示使用
  final String helperText;
  final TextStyle helperStyle;
  final String labelText; //显示在输入框内的提示语，一旦输入框获取焦点就字体缩小并上移到输入上方，作为提示使用
  final TextStyle labelStyle;
  final bool labelFloating; //默认为true，表示labelText是否上浮，true上浮，false表示获取焦点后labelText消失
  final bool isDense; //是否开启紧密
  final bool filled; //输入框颜色
  final bool enabled; //开始输入
  final InputBorder disabledBorder; //不能输入状态框
  final InputBorder enabledBorder; //开启输入状态框
  final InputBorder errorBorder; //失去焦点时，error时下划线显示的边框样式，不设置则使用默认的的下划线
  final InputBorder focusedErrorBorder; //获取焦点时，error时下划线显示的边框样式，不设置则使用默认的的下划线
  final InputBorder focusedBorder; //获取焦点输入框
  final InputBorder defaultBorder; //以上都未设置时 默认输入框

  final Color fillColor;
  final TextStyle hintStyle;
  final String hintText;
  final int hintMaxLines;
  final EdgeInsetsGeometry contentPadding;
  final TextInputAction textInputAction;
  final double cursorWidth;
  final Radius cursorRadius;
  final Brightness keyboardAppearance;

  //长按输入的文字时，true显示系统的粘贴板  false不显示
  final bool enableInteractiveSelection;

// 控制键盘大小写切换的   试过了 但是好像没有效果？？
// textCapitalization: TextCapitalization.characters,  // 输入时键盘的英文都是大写
// textCapitalization: TextCapitalization.none,  //  键盘英文默认显示小写
// textCapitalization: TextCapitalization.sentences, // 在输入每个句子的第一个字母时，键盘大写形式，输入后续字母时键盘小写形式
// textCapitalization: TextCapitalization.words,// 在输入每个单词的第一个字母时，键盘大写形式，输入其他字母时键盘小写形式
  final TextCapitalization textCapitalization;

// 自定义数字显示   指定maxLength后 右下角会出现字数，flutter有默认实现  可以通过这个自定义
  final InputCounterWidgetBuilder buildCounter;

  TextInputField({
    Key key,
    int maxLines,
    TextAlign textAlign,
    bool obscureText,
    bool autoFocus,
    bool enabled,
    bool enableInteractiveSelection,
    bool maxLengthEnforced,
    TextInputAction textInputAction,
    TextDirection textDirection,
    Color cursorColor,
    Radius cursorRadius,
    double cursorWidth,
    Brightness keyboardAppearance,
    TextCapitalization textCapitalization,
    this.icon,
    this.controller,
    this.inputDecoration,
    this.onSubmitted,
    this.inputStyle,
    this.keyboardType,
    this.minLines,
    this.maxLength,
    this.onChanged,
    this.hintText,
    this.hintStyle,
    this.contentPadding,
    this.inputFormatter,
    this.fillColor,
    this.inputTextType,
    this.isDense,
    this.disabledBorder,
    this.enabledBorder,
    this.focusedBorder,
    this.defaultBorder,
    this.labelText,
    this.labelStyle,
    this.prefixText,
    this.prefixIcon,
    this.prefix,
    this.prefixStyle,
    this.suffixIcon,
    this.suffixText,
    this.suffix,
    this.suffixStyle,
    this.errorBorder,
    this.focusedErrorBorder,
    this.onTap,
    this.filled,
    this.errorStyle,
    this.errorText,
    this.errorMaxLines,
    this.helperText,
    this.helperStyle,
    this.labelFloating,
    this.focusNode,
    this.header,
    this.footer,
    this.hintMaxLines,
    this.counterText,
    this.counterStyle,
    this.counter,
    this.buildCounter,
    this.onEditingComplete,
  })
      : this.obscureText = obscureText ?? false,
  //键盘大小写的显示 Only supports text keyboards  但是好像不起作用？
  //characters 默认为每个字符使用大写键盘
  //sentence 默认为每个句子的第一个字母使用大写键盘
  //word 默认为每个单词的第一个字母使用大写键盘。
  //none 默认使用小写
        this.textCapitalization = textCapitalization ?? TextCapitalization.none,
  //长按输入的文字时，true显示系统的粘贴板  false不显示
        this.enableInteractiveSelection = enableInteractiveSelection ?? true,
  //自定义数字显示   指定maxLength后 右下角会出现字数，flutter有默认实现  可以通过这个自定义
  //光标颜色
        this.cursorColor = cursorColor ?? getColors(black70),
  //光标圆角
        this.cursorRadius = cursorRadius ?? Radius.circular(1),
  //光标宽度
        this.cursorWidth = cursorWidth ?? 2,
  // 键盘外观  仅ios有效
        this.keyboardAppearance = keyboardAppearance,
//      默认true  超过长度后输入无效  右下角数字 显示10/10   此时onchange方法依然会调用，返回值就是限制了长度的值 超过后的输入不显示
//      false 超过后可继续输入  右下角数字显示，比如 23/10
        this.maxLengthEnforced = maxLengthEnforced ?? true,
//       设置键盘上enter键的显示内容
//       textInputAction: TextInputAction.search, //搜索
//       textInputAction: TextInputAction.none,//默认回车符号
//       textInputAction: TextInputAction.done,//安卓显示 回车符号
//       textInputAction: TextInputAction.go,//开始
//       textInputAction: TextInputAction.next,//下一步
//       textInputAction: TextInputAction.send,//发送
//       textInputAction: TextInputAction.continueAction,//android  不支持
//       textInputAction: TextInputAction.emergencyCall,//android  不支持
//       textInputAction: TextInputAction.newline,//安卓显示 回车符号
//       textInputAction: TextInputAction.route,//android  不支持
//       textInputAction: TextInputAction.join,//android  不支持
//       textInputAction: TextInputAction.previous,//安卓显示 回车符号
//       textInputAction: TextInputAction.unspecified,//安卓显示 回车符号
        this.textInputAction = textInputAction ?? TextInputAction.done,
        this.autoFocus = autoFocus ?? false,
        this.maxLines = maxLines ?? 1,
  //从左边输入  光标在左边
  //从右边输入  光标在右边
//      this.textDirection = textDirection ?? TextDirection.rtl,
        this.textDirection = textDirection ?? TextDirection.ltr,
        this.enabled = enabled ?? true,
        this.textAlign = textAlign ?? TextAlign.left,
        super(key: key);

//  解决切换后台 再切换前台输入框为null
//  @override
//  void initState() {
//    super.initState();
//    WidgetsBinding.instance.addObserver(this);
//    controller = widget.controller ?? TextEditingController();
//    setController(widget.value);
//  }
//
//  setController(String value) {
//    if (value.length > 0) {
//      controller.text = value;
//      解决光标左移问题
//      controller.selection = TextSelection.fromPosition(TextPosition(offset: value.length));
//    }
//  }
//
//  @override
//  void didChangeAppLifecycleState(AppLifecycleState state) {
//    super.didChangeAppLifecycleState(state);
//    switch (state) {
//      case AppLifecycleState.inactive: //处于这种状态的应用程序应该假设它们可能在任何时候暂停。前台
//        break;
//      case AppLifecycleState.paused: // 应用程序不可见，后台  切换后台
//        break;
//      case AppLifecycleState.resumed: // 应用程序可见，前台  从后台切换前台
//        setController(text);
//        break;
//      case AppLifecycleState.detached: // 申请将暂时暂停
//        break;
//    }
//  }
//
//  @override
//  void dispose() {
//    WidgetsBinding.instance.removeObserver(this);
//    super.dispose();
//  }

  @override
  Widget build(BuildContext context) {
    return header != null || footer != null
        ? Column(mainAxisSize: MainAxisSize.min, children: <Widget>[
      //输入框头部
      Offstage(offstage: header == null, child: header),
      textField(context),
      //输入框底部
      Offstage(offstage: footer == null, child: footer)
    ])
        : textField(context);
  }

  Widget textField(BuildContext context) {
    return TextField(
      //长按输入的文字时，true显示系统的粘贴板  false不显示
      enableInteractiveSelection: enableInteractiveSelection,
      //自定义数字显示   指定maxLength后 右下角会出现字数，flutter有默认实现  可以通过这个自定义
      buildCounter: buildCounter,
      maxLines: maxLines,
      minLines: minLines,
      maxLength: maxLength,
      maxLengthEnforced: maxLengthEnforced,
      keyboardAppearance: keyboardAppearance,
      textDirection: textDirection,
      textCapitalization: textCapitalization,
      textAlign: textAlign,
      focusNode: focusNode,
      autofocus: autoFocus,
      textInputAction: textInputAction,
      inputFormatters: inputFormatter ?? inputType(),
      cursorColor: cursorColor,
      cursorRadius: cursorRadius,
      cursorWidth: cursorWidth,
      obscureText: obscureText,
      keyboardType: keyboardType,
      style: inputStyle ?? WayStyles.textStyleBlack70(fontSize: 16),
      controller: controller,
      decoration: inputDecoration ??
          InputDecoration(
            //显示在输入框前面的图标，在文字和下划线前面
            icon: icon,
            //输入框光标前面的图标和文字
            prefixIcon: prefixIcon,
            //显示在输入框内，光标前面的图标，注意icon是在输入框外
            prefixText: prefixText,
            //显示在输入框内，光标前面的文字   获得焦点后显示
            prefixStyle: prefixStyle,
            //显示在输入框内，光标前面的指定图片或其他Widget   获取焦点后显示。注意prefix不能与prefixIcon prefixText同时指定
            prefix: prefix,

            //输入框光标后面的图标和文字
            suffixIcon: suffixIcon,
            //显示在输入框内，光标文字后面，输入框最后面的图标
            //text无点击事件
            suffixText: suffixText,
            //显示在输入框内，输入框最后面的文字    但是在suffixIcon之前。 注意suffixText不能与suffix共存
            suffixStyle: suffixStyle,
            //显示在输入框内，最后面的指定图片或其他Widget
            suffix: suffix,

            // 显示在输入的下划线外右下角的文字提示
            counterText: counterText,
            //显示在输入的下划线外右下角的文字提示,会覆盖maxLength的右下角显示的字数限制。
            counterStyle: counterStyle,
            //显示在输入的下划线外右下角的提示,可以是任何Widget ，counterText与counter只能存在一个
            counter: counter,

            // 输入框的文字提示
            labelText: labelText,
            //显示在输入框内的提示语，一旦输入框获取焦点就字体缩小并上移到输入上方，作为提示使用
            labelStyle: labelStyle,
            hasFloatingPlaceholder: labelFloating,
            //默认为true，表示labelText是否上浮，true上浮，false表示获取焦点后labelText消失
            helperText: helperText,
            //显示在输入框下划线下面的提示语，提示使用
            helperStyle: helperStyle,

            hintText: hintText,
            //显示在输入框内的提示信息，当输入框为空时显示，一旦开始输入内容就消失
            hintStyle: hintStyle ?? WayStyles.textStyleBlack30(fontSize: 16),
            hintMaxLines: hintMaxLines ?? 1,
            //提示语的做多显示行数，超过行数显示...

            // 错误提示相关
            errorText: errorText,
            //在输入框下方的提示语，通常用于错误提示，比如密码错误 用户名错误等  同时输入框的下划线修改为了红色
            errorStyle: errorStyle,
            errorMaxLines: errorMaxLines,
            //做多显示的行数  ，超过行数显示...
            errorBorder: errorBorder,
            //失去焦点时，error时下划线显示的边框样式，不设置则使用默认的的下划线
            focusedErrorBorder: focusedErrorBorder,
            //获取焦点时，error时下划线显示的边框样式，不设置则使用默认的的下划线
            //输入框内文字 密集显示
            isDense: isDense ?? false,
            //改变输入框是否为密集型，默认为false，修改为true时，图标及间距会变小  行间距减小
            //内部padding
            contentPadding: contentPadding ?? EdgeInsets.zero,
            //输入框的padding  对内部的文字有效
            //背景色
            fillColor: fillColor,
            //输入框内部的填充颜色  需filled=true
            filled: filled,
            //输入框禁用时，下划线的样式.如果设置了errorText，则此属性无效
            disabledBorder: disabledBorder,
            //输入框启用时，下划线的样式
            enabledBorder: enabledBorder,
            //是否启用输入框
            enabled: enabled,
            //获取焦点时，下划线的样式
            focusedBorder: focusedBorder,
            //级别最低的border，没有设置其他border时显示的border
            border: defaultBorder,
//            border: InputBorder.none,
          ),
      onChanged: onChanged,
      onTap: onTap,
      onSubmitted: onSubmitted,
      enabled: enabled,
      //按回车时调用 先调用此方法  然后调用onSubmitted方法
      onEditingComplete: onEditingComplete,
    );
  }

  List<TextInputFormatter> inputType() {
    switch (inputTextType) {
      case InputTextType.number:
        return [WhitelistingTextInputFormatter.digitsOnly];
      case InputTextType.password:
        return [
          WhitelistingTextInputFormatter(RegExp(WayConstant.regExpPassword))
        ]; //密码常见类型
      case InputTextType.decimal:
        return [
          WhitelistingTextInputFormatter(RegExp(WayConstant.regExpDecimal))
        ]; //只允许输入小数
      case InputTextType.letter:
        return [
          WhitelistingTextInputFormatter(RegExp(WayConstant.regExpLetter))
        ]; //只允许输入字母
      case InputTextType.chinese:
        return [
          WhitelistingTextInputFormatter(RegExp(WayConstant.regExpChinese))
        ]; //只允许输入汉字
      case InputTextType.email:
        return [
          WhitelistingTextInputFormatter(RegExp(WayConstant.regExpEmail))
        ]; //只允许输入邮箱
      case InputTextType.phone:
        return [
          WhitelistingTextInputFormatter(RegExp(WayConstant.regExpPhone))
        ]; //只允许输入国内电话号
      case InputTextType.mobilePhone:
        return [
          WhitelistingTextInputFormatter(RegExp(WayConstant.regExpMobilePhone))
        ]; //只允许输入国内手机号
      case InputTextType.idCard:
        return [
          WhitelistingTextInputFormatter(RegExp(WayConstant.regExpIdCard))
        ]; //只允许输入身份证
      case InputTextType.ip:
        return [
          WhitelistingTextInputFormatter(RegExp(WayConstant.regExpIP))
        ]; //只允许输入IP
      case InputTextType.positive:
        return [
          WhitelistingTextInputFormatter(RegExp(WayConstant.regExpPositive))
        ]; //只允许输入正数
      case InputTextType.negative:
        return [
          WhitelistingTextInputFormatter(RegExp(WayConstant.regExpNegative))
        ]; //只允许输入负数
      case InputTextType.text:
        return inputFormatter;
      default:
        return inputFormatter;
    }
  }
}
